<?php

/**
 * @file
 * BucketJobManager.
 */

namespace Drupal\tmgmt_smartling\Smartling;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\tmgmt_extension_suit\Utils\UniqueQueueItem;
use Psr\Log\LoggerInterface;
use Smartling\Jobs\JobStatus;

/**
 * Class BucketJobManager.
 */
class BucketJobManager {

  /**
   * @var \Drupal\tmgmt_smartling\Smartling\SmartlingApiWrapper
   */
  private $api_wrapper;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  private $module_handler;

  /**
   * @var \Drupal\tmgmt_extension_suit\Utils\UniqueQueueItem
   */
  private $unique_queue_item_util;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * BucketJobManager constructor.
   *
   * @param \Psr\Log\LoggerInterface $logger
   * @param \Drupal\tmgmt_smartling\Smartling\SmartlingApiWrapper $api_wrapper
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   * @param \Drupal\tmgmt_extension_suit\Utils\UniqueQueueItem $unique_queue_item_util
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   */
  public function __construct(LoggerInterface $logger, SmartlingApiWrapper $api_wrapper, ModuleHandlerInterface $module_handler, UniqueQueueItem $unique_queue_item_util, ConfigFactoryInterface $config_factory) {
    $this->logger = $logger;
    $this->api_wrapper = $api_wrapper;
    $this->module_handler = $module_handler;
    $this->unique_queue_item_util = $unique_queue_item_util;

    // Pick up first available Smartling provider settings.
    $translator_ids = $config_factory->listAll('tmgmt.translator');

    foreach ($translator_ids as $id) {
      $config = $config_factory->get($id);

      if ($config->get('plugin') === 'smartling') {
        $raw_data = $config->getRawData();
        $this->api_wrapper->setSettings($raw_data['settings']);

        break;
      }
    }
  }

  /**
   * @param string $suffix
   * @return string
   */
  private function getName($suffix = '') {
    $date = date('m/d/Y');
    $name = "Daily Bucket Job $date";
    $this->module_handler->alter('tmgmt_smartling_bucket_job_name', $name);

    return $name . $suffix;
  }

  /**
   * @param array $jobs
   */
  public function handle(array $jobs) {
    $job_uid = NULL;
    $job_name = $this->getName();
    $response = $this->api_wrapper->listJobs($job_name, [
      JobStatus::AWAITING_AUTHORIZATION,
      JobStatus::IN_PROGRESS,
      JobStatus::COMPLETED,
    ]);

    // Try to find the latest created bucket job.
    if (!empty($response['items'])) {
      $job_uid = $response['items'][0]['translationJobUid'];
    }

    // If there is no existing bucket job then create new one.
    if (empty($job_uid)) {
      $job_uid = $this->api_wrapper->createJob($job_name, (string) t('Bucket job: contains updated content.'));

      // If there is a CANCELED/CLOSED bucket job then we have to come with new
      // job name in order to avoid "Job name is already taken" error.
      if (empty($job_uid)) {
        $job_name = $this->getName(' ' . date('H:i:s'));
        $job_uid = $this->api_wrapper->createJob($job_name, (string) t('Bucket job: contains updated content.'));
      }
    }

    if (empty($job_uid)) {
      $this->logger->error(t("Queueing file upload into the bucket job failed: can't find/create job."));

      return;
    }

    $batch_uid = $this->api_wrapper->createBatch($job_uid, FALSE);

    if (empty($batch_uid)) {
      $this->logger->error(t("Queueing file upload into the bucket job failed: can't create batch."));

      return;
    }

    foreach ($jobs as $job) {
      $last_job = end($jobs);

      $job->set('settings', [
        'batch_uid' => $batch_uid,
        'batch_execute_on_job' => ($last_job->id() == $job->id()) ? $job->id() : FALSE,
      ]);
      $job->save();
    }
  }

}
